;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x63813dca304f95ae7c4c51ea48f91a0f . "./code.cg2js/cg-acache.fasl")
(#x4bdf120f945354652ddd4beb0f47e3d1 . "./code.cg2js/cg-base.fasl")
(#xbb34e52fc96ddb6ee89680281e1706b7 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x44eb7783159fcd844120f35a1e551208 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xe566fde76d02a540aaec035eeea9e1b8 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xbf553b1527a637d9da3f491f0189461d . "./code.cg2js/cg-button.fasl")
(#x540d5348b64737e355c76413fee2ea5e . "./code.cg2js/cg-calendar.fasl")
(#x1d3b072ee7bc26c76e1def69b92973d7 . "./code.cg2js/cg-caret.fasl")
(#xb1de5f2c2f27d4665bc6ebff6792de21 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x8b8bb7e2a8ac7b9c4f837104a23213b5 . "./code.cg2js/cg-chart-widget.fasl")
(#x1eb238c796e068a0c825bc46e628b3f1 . "./code.cg2js/cg-check-box.fasl")
(#x20345d4ae0b6aa321096602aa86debd8 . "./code.cg2js/cg-choice-list.fasl")
(#x873589e0299a4810396638fc663d2a1c . "./code.cg2js/cg-choose-printer.fasl")
(#x356a4f119ea572658cfc4b692f30ba9a . "./code.cg2js/cg-class-grid.fasl")
(#x823b6e065f84a686700a3947066500b8 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xa2a852f8c6d9949ea793d344000b37d8 . "./code.cg2js/cg-class-support.fasl")
(#x96abda0531838742e3830b46c0134e91 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x3f64c08b2d01eabb7e42c8790566029a . "./code.cg2js/cg-clipboard-stack.fasl")
(#xe71bd7ae7546e6b78c0cfef4c37958b6 . "./code.cg2js/cg-clipboard.fasl")
(#xfe14b0738629fe82929535dce4a3f0e2 . "./code.cg2js/cg-color-dialog.fasl")
(#xc4728125d3a1854bc9aec8dcd2251064 . "./code.cg2js/cg-combo-box.fasl")
(#xf19426c22496bc2f874951a4df4fd607 . "./code.cg2js/cg-common-control.fasl")
(#x584cdaa18392c645e6d6faaf63dcdda9 . "./code.cg2js/cg-comtab.fasl")
(#xaf5c359b31d35e86cc934b5bb2adc9b7 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xfd2dfbf72809459b69ba1c9da85e12d2 . "./code.cg2js/cg-curve.fasl")
(#x4788ca98508d10c6222980c0120729ea . "./code.cg2js/cg-dde-utils.fasl")
(#xc9da52f2ce018cd14398f81a3842f793 . "./code.cg2js/cg-dev.fasl")
(#x805cc92a416ade7dbfe5195d5520e2ed . "./code.cg2js/cg-dialog-item.fasl")
(#x3fc9f7b451539bac026234568244f288 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xae37ef2cceda655c5d79ee4fe6847554 . "./code.cg2js/cg-directory-dialog.fasl")
(#x3df3bbd4e4df2096498b16a910f9b2a8 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xda833f51038e132b49b8f234fb7cc15c . "./code.cg2js/cg-drag-and-drop.fasl")
(#x6b500db7ee68498b76eb59ede9157cb2 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x523b7cd5fe1b42d0d1147be684000f16 . "./code.cg2js/cg-drawable.fasl")
(#xc0725ee79098113d8c7a6b6030889ae7 . "./code.cg2js/cg-dropping-outline.fasl")
(#x2056e8fbb1f9a093f80ef424bc386730 . "./code.cg2js/cg-edit-in-place.fasl")
(#xd2b32935482d9df451b677d3021fd92c . "./code.cg2js/cg-editable-text.fasl")
(#xe7535eb4eaa26e0ffb9f08a788866f2f . "./code.cg2js/cg-file-dialog.fasl")
(#x0f86213f48026a345a3d42b003e97eb6 . "./code.cg2js/cg-fill-texture.fasl")
(#x951b14d4f49fb975250c439af7d43b5c . "./code.cg2js/cg-find-string-dialog.fasl")
(#xe06610334b867ec83ba1fc37bff9b7b1 . "./code.cg2js/cg-font-dialog.fasl")
(#x05e120b957826fc8bd81dc63e2ea89cc . "./code.cg2js/cg-gesture-emulation.fasl")
(#x4e14d9017a48d616ea0772905cbf258b . "./code.cg2js/cg-get-pixmap.fasl")
(#x0e198fe4023af4493b5b52f02a27e2ba . "./code.cg2js/cg-get-position.fasl")
(#xc14f303197621a50e4e5f9d7bbd651a8 . "./code.cg2js/cg-graphics-context.fasl")
(#x75a8a150dd762b2e0af2ae0891415cb7 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xae8e0ffc2ed6b84fd3b454ea3d3b6160 . "./code.cg2js/cg-grid-widget.fasl")
(#x3b13159107975a8344f06ffac0351590 . "./code.cg2js/cg-group-box.fasl")
(#xfaa2a554e71d02ea8253f75c93b8c38d . "./code.cg2js/cg-header-control.fasl")
(#x252adb3973443981a3815cba036b21a4 . "./code.cg2js/cg-hotspot.fasl")
(#x645fde6e830696f838bf51a04dbbf0ef . "./code.cg2js/cg-html-dialog.fasl")
(#xf305c0c964816818e8d14b6a57f015b5 . "./code.cg2js/cg-html-widget.fasl")
(#xf0cfea1050252594437956e9f98cc46f . "./code.cg2js/cg-icon-pixmap.fasl")
(#x1a8935276c932c0c99d2c688d8f4aad0 . "./code.cg2js/cg-icon.fasl")
(#x2aa1991468a70277ee81cfd9e69f4dd5 . "./code.cg2js/cg-ie.fasl")
(#x9d3119f64392459f0c147b7c524851d3 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x4509fb1456c6d6d2c42ca195b65b912e . "./code.cg2js/cg-intersect.fasl")
(#xc21eb8df9a74c1ae17b6be04ab7d5882 . "./code.cg2js/cg-item-list.fasl")
(#xd50dced8a4c660562849819e83f8026d . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x214ef42f71532087cb926180ea467969 . "./code.cg2js/cg-lamp.fasl")
(#x6937a5da7097a3f4a8ee0c83fc5f6aaf . "./code.cg2js/cg-layout.fasl")
(#xc0bfa21459927916536b8c33a34533b2 . "./code.cg2js/cg-lettered-menu.fasl")
(#xab01033500c5b578ff61a78fd6955c24 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x0e5c8978ab0bca1adcc879abb125131d . "./code.cg2js/cg-lisp-text.fasl")
(#xa535a62340e6a203ddbd4ab3e139d7e0 . "./code.cg2js/cg-lisp-widget.fasl")
(#x56975628be97ce8d0871d376e68df8d6 . "./code.cg2js/cg-list-view.fasl")
(#x6168d25bde02323211c67a8fca0562f7 . "./code.cg2js/cg-mci.fasl")
(#xb099b588bebbe6f6822969f5a9808f98 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x0cc6b7316e4633c0fc5fa2b09866dfe2 . "./code.cg2js/cg-menu.fasl")
(#x5747e751b90c379c4c62b0fff40c85b3 . "./code.cg2js/cg-message-dialog.fasl")
(#xedbbec1d3395e7adf16be439e1e1ca1b . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf54ba7f64dc913f1223d8f3c1d7278aa . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xa5986c05dfac2df74804a7726d35d48d . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xc9001e275047311fecbfd70c0756f077 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x9e8b721945cd6be8f8b1face8f57bb36 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x2fbfe5bb0a38a108604a9c844a152c1e . "./code.cg2js/cg-nodes.fasl")
(#x400d23e2f6688c0aa951224e660e7df5 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xff8ce3d159e1eba2bfd8097a01fb9361 . "./code.cg2js/cg-object-editor.fasl")
(#x35cc2a6f16763b7ae2458d73a8da53f2 . "./code.cg2js/cg-ocx.fasl")
(#x1db4021dc580c5cead252bef5d7cddc4 . "./code.cg2js/cg-os-widget.fasl")
(#x6567d2f43d5ffca94eecb2d51ac7a545 . "./code.cg2js/cg-os-window.fasl")
(#x0249d898e95891359df26c9b465242b3 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x12e4f7ce4520d98e3980a5c834c53424 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xac953d3fdfd131b248179bf9b86b43f7 . "./code.cg2js/cg-outline.fasl")
(#xc687637f18cedc382dd4daef96ee35f1 . "./code.cg2js/cg-palette.fasl")
(#x639ab81242a8976ba03dfc12eedc88b5 . "./code.cg2js/cg-paren-matching.fasl")
(#xbaec86acabad436793391bb7f98db986 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x1dfb1833747af37ed50c513bc25d2598 . "./code.cg2js/cg-picture-widget.fasl")
(#xdb14c6bccc5d8568d1fa6a6e6f435aad . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x213958a5accf092d653d81fe38bdd84d . "./code.cg2js/cg-pixmap-printing.fasl")
(#x7cae2130e83000356b330c57f5279424 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x4407d89974acc5022860c4ff4ab407f7 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x8a7055858dee2475b57acad3047bbec3 . "./code.cg2js/cg-pixmap.fasl")
(#x654b35ee789b54e9318038ff131bb1c0 . "./code.cg2js/cg-player.fasl")
(#x08695fc56825e5451aca051a3d13d961 . "./code.cg2js/cg-plot-widget.fasl")
(#xb477f70736b179e68440fa07019e8ac0 . "./code.cg2js/cg-printing.fasl")
(#x666987ee74980b1123908bdd86f2ea43 . "./code.cg2js/cg-progress-indicator.fasl")
(#x38e53e98fb474d28d4a4f3c7e40985a1 . "./code.cg2js/cg-project-window.fasl")
(#x9bcf9abe0961fc676749782a5fcbb44c . "./code.cg2js/cg-property.fasl")
(#x48864ec1c6ef82fb3577ecbc8292fb6d . "./code.cg2js/cg-radio-button.fasl")
(#x9b8bb4af2ee45708386b2edcf0850c49 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x0df55e90e577474f1ebc7415f75592c6 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x56759bd02d5e02f10e0b099a9c45558d . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x91d9647a7a5771f5cb77a1ebc127274b . "./code.cg2js/cg-rich-edit.fasl")
(#x27097a0eed48a189b0f243d56f85e4d8 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x8240faf7f7ef771e0b37fd930978c325 . "./code.cg2js/cg-scaling-stream.fasl")
(#x9ea95fef783920521d8a236525ee9236 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xf4d9ce9f2fcc27b21711460190305a96 . "./code.cg2js/cg-scroll-bar.fasl")
(#xefa7d4c473792afc9ecb84ef12a74904 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x1675929a16fa81543793402b7a81efae . "./code.cg2js/cg-selected-object.fasl")
(#xc563353d82cb04ac6d025453888fb76e . "./code.cg2js/cg-shortcut-menu.fasl")
(#xa706b9ae595efe0be6e1a40fcb99edfe . "./code.cg2js/cg-split-bar.fasl")
(#x92695d432e9b0b939f5312d0e2c2a1b7 . "./code.cg2js/cg-static-text.fasl")
(#xf1c3d6cee1bcac85aa25fa9eb59484ea . "./code.cg2js/cg-status-bar.fasl")
(#x45fbf6f617cec0e0e1393b6d366e6e26 . "./code.cg2js/cg-string-dialog.fasl")
(#xb7e90efe788a6e1fcf7b1adb70228d02 . "./code.cg2js/cg-tab-control.fasl")
(#xbd3bf525a892ea8fb9a7bb5203d898d6 . "./code.cg2js/cg-template-string.fasl")
(#xc44088eba9d2ef0b2c92002e332ffe2a . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x4efa9141ee0e55ec61298fda46f69f1c . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xbdd557e5cab0634150a8313fc80dd0d0 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xc79af49fdee4088950372cca535f24dc . "./code.cg2js/cg-text-or-combo.fasl")
(#x609ac2e4c0fa97bd67c51a488a141fd9 . "./code.cg2js/cg-text-widget.fasl")
(#xc140d2fc89276e333a670cf44d3d8be1 . "./code.cg2js/cg-timer.fasl")
(#x04be5e82e5163d1eea9dbef471e54991 . "./code.cg2js/cg-toggling-widget.fasl")
(#xdc57af91e5e5ff998a65b0574b7f2d67 . "./code.cg2js/cg-toolbar.fasl")
(#xa88442d34d0c6cf6979cbf0d65ea5d82 . "./code.cg2js/cg-tooltip.fasl")
(#xd30f63c210ea014fecc2eaa914ced27d . "./code.cg2js/cg-touch.fasl")
(#x4b99ec453ae28541e8a762388bfa2df4 . "./code.cg2js/cg-trackbar.fasl")
(#x43b7969a6dc9ff69b14f7842869bc63e . "./code.cg2js/cg-tray.fasl")
(#x8c0541a3600635c7a81c8b0e52a49ec2 . "./code.cg2js/cg-up-down-control.fasl")
(#x629629e958895714dbae5b050c054df1 . "./code.cg2js/cg-utility-dialog.fasl")
(#xa20a655dc4a490dc71f5f07834d1655a . "./code.cg2js/cg-web-browser-dde.fasl")
(#x1f4b0555925712817445b59426482ba2 . "./code.cg2js/cg-web-browser.fasl")
(#x221462ce1ad47c3015983745c0bee756 . "./code.cg2js/cg-wrap-string.fasl")
(#x6b2c24e0e9a5f70c0da065a9d4ff8a44 . "./code.cg2js/cg-yes-no-list.fasl")
(#x53cdbeac7da216336ec369c80d18c301 . "./code.cg2js/cg.fasl")
(#x5e6cbcccf9c1469e19a60765bad32c38 . "./code.cg2js/cg-yes-no-string.fasl")
(#x5c1b24819303f1ea0e8651ae126788cb . "./code.idejs/ide-acache.fasl")
(#xcb8cafd8c6de14e9e4f0cf129f5792d3 . "./code.idejs/ide-apropos.fasl")
(#x88179bf99ee1ab15ef5d6ac610cc6906 . "./code.idejs/ide-base.fasl")
(#xd9a5767e843da4a0a3b6db83098e106c . "./code.idejs/ide-class-browser.fasl")
(#xa6185aa1fdefc7f201b0f67d4f967b67 . "./code.idejs/ide-class-grapher.fasl")
(#x02fec191d18813b1b893f10eededa66c . "./code.idejs/ide-clipboard.fasl")
(#xcacbb95851e5af55da6712add856ee91 . "./code.idejs/ide-dev.fasl")
(#x1030bb59fbc1b83737568639973bada7 . "./code.idejs/ide-editor.fasl")
(#xd248a68ddf1878c7591a986bebc333d0 . "./code.idejs/ide-find-definitions.fasl")
(#xc8f3e6ddcb3d59a7063a15146373cf21 . "./code.idejs/ide-find-in-files.fasl")
(#xfd03db1f439239c79d50b111043579ed . "./code.idejs/ide-form.fasl")
(#xa12cc3ef4ad2f01953702c18b028e4cd . "./code.idejs/ide-grapher.fasl")
(#x608976fc1286649506414ef9549ebb7f . "./code.idejs/ide-inspector.fasl")
(#x252c579d10ad8815c0a699806885e253 . "./code.idejs/ide-intro.fasl")
(#x64918f85ec505e0359cd578de3ca87a0 . "./code.idejs/ide-keyshort.fasl")
(#xd89701c70d60c9564c4771b25b5ea777 . "./code.idejs/ide-navigator.fasl")
(#x60e15b9c47fdae90936060895938e167 . "./code.idejs/ide-package-list.fasl")
(#xfb92fa9a71d24a9a4c89a1f6f1b9902b . "./code.idejs/ide-patch-dialog.fasl")
(#xd7a01f48b022c9fde092b78a85b87e51 . "./code.idejs/ide-process-browser.fasl")
(#xe973044c96d46e82076011d04bc91756 . "./code.idejs/ide-project.fasl")
(#x720598879530d3279bf62aca666834a0 . "./code.idejs/ide-trace.fasl")
(#x0716ef920f68e5e90435fc19b0766e22 . "./code.idejs/ide-window-list.fasl")
(#x939fbbb4c22044acf54e31d53faa7449 . "./code.idejs/ide.fasl")
(#xf82fd41d79afb1da19d111da8345005f . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
